<?php

namespace Drupal\payment_stripe\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\payment_stripe\Event\StripeEvents;
use Drupal\payment_stripe\Event\StripeWebhookChargeFailed;
use Drupal\payment_stripe\Event\StripeWebhookChargeSucceeded;
use Drupal\payment_stripe\Event\StripeWebhookEvent;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller routines for book routes.
 */
class StripeWebhookController extends ControllerBase {

  /**
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $event_dispatcher;

  /**
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $event_dispatcher
   */
  public function __construct(EventDispatcherInterface $event_dispatcher) {
    $this->event_dispatcher = $event_dispatcher;
  }

  /**
   * When this controller is created, it will get the di_example.talk service
   * and store it.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   * @return static
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('event_dispatcher')
    );
  }
  /**
   * Handle webhook.
   */
  function handle(Request $request) {
    $config = \Drupal::config('payment_stripe.settings');

    \Stripe\Stripe::setApiKey($config->get('apikey.' . $config->get('environment') . '.secret'));

    $environment = $config->get('environment');
    $payload = @file_get_contents("php://input");
    $signature = $request->server->get('HTTP_STRIPE_SIGNATURE');
    $secret = $config->get("apikey.$environment.webhook");

    try {
      if (!empty($secret)) {
        $event = \Stripe\Webhook::constructEvent($payload, $signature, $secret);
      }
      else {
        $data = json_decode($payload, true);
        $jsonError = json_last_error();
        if ($data === null && $jsonError !== JSON_ERROR_NONE) {
            $msg = "Invalid payload: $payload "
              . "(json_last_error() was $jsonError)";
            throw new \UnexpectedValueException($msg);
        }

        if ($environment == 'live') {
          $event = \Stripe\Event::retrieve($data['id']);
        }
        else {
          $event = \Stripe\Event::constructFrom($data, null);
        }
      }
    }
    catch(\UnexpectedValueException $e) {
      return new Response('Invalid payload', Response::HTTP_BAD_REQUEST);
    }
    catch(\Stripe\Error\SignatureVerification $e) {
      return new Response('Invalid signature', Response::HTTP_BAD_REQUEST);
    }

    // Dispatch the webhook event.
    $this->event_dispatcher
      ->dispatch(StripeEvents::WEBHOOK, new StripeWebhookEvent($event));

    // Query DB and modify Payment if it is necessary according webhook type.
    switch ($event->type) {
      case 'charge.succeeded':
        \Drupal::logger($event->type)->notice('Webhook for ' . $event->type);
        break;
      case 'charge.failed':
        \Drupal::logger($event->type)->notice('Webhook for ' . $event->type);
        break;
    }

    return new Response('OK', Response::HTTP_OK);
  }

  /**
   * chargeSucceeded webhook.
   *
   * @param Request $request
   * @return Response
   */
  function chargeSucceeded (Request $request) {
    $config = \Drupal::config('payment_stripe.settings');

    \Stripe\Stripe::setApiKey($config->get('apikey.' . $config->get('environment') . '.secret'));

    $environment = $config->get('environment');
    $payload = @file_get_contents('php://input');
    $signature = $request->server->get('HTTP_STRIPE_SIGNATURE');
    $secret = $config->get("apikey.$environment.webhook");

    try {
      if (!empty($secret)) {
        $event = \Stripe\Webhook::constructEvent($payload, $signature, $secret);
      } else {
        $data = json_decode($payload, true);
        $jsonError = json_last_error();
        if ($data === null && $jsonError !== JSON_ERROR_NONE) {
          $msg = "Invalid payload: $payload "
            . "(json_last_error() was $jsonError)";
          throw new \UnexpectedValueException($msg);
        }

        if ($environment == 'live') {
          $event = \Stripe\Event::retrieve($data['id']);
        } else {
          $event = \Stripe\Event::constructFrom($data, null);
        }
      }
    } catch (\UnexpectedValueException $e) {
      return new Response('Invalid payload', Response::HTTP_BAD_REQUEST);
    } catch (\Stripe\Error\SignatureVerification $e) {
      return new Response('Invalid signature', Response::HTTP_BAD_REQUEST);
    }

    // Dispatch the webhook charge succeeded event.
    $this->event_dispatcher
      ->dispatch(StripeEvents::WEBHOOK_CHARGE_SUCCEEDED, new StripeWebhookChargeSucceeded($event));

    \Drupal::logger('chargeSucceeded')->notice('Charge succeeded' . $event->type);


    return new Response('OK', Response::HTTP_OK);
  }

  /**
   * chargeFailed webhook.
   *
   * @param Request $request
   * @return Response
   */
  function chargeFailed (Request $request) {
    $config = \Drupal::config('payment_stripe.settings');

    \Stripe\Stripe::setApiKey($config->get('apikey.' . $config->get('environment') . '.secret'));

    $environment = $config->get('environment');
    $payload = @file_get_contents('php://input');
    $signature = $request->server->get('HTTP_STRIPE_SIGNATURE');
    $secret = $config->get("apikey.$environment.webhook");

    try {
      if (!empty($secret)) {
        $event = \Stripe\Webhook::constructEvent($payload, $signature, $secret);
      } else {
        $data = json_decode($payload, true);
        $jsonError = json_last_error();
        if ($data === null && $jsonError !== JSON_ERROR_NONE) {
          $msg = "Invalid payload: $payload "
            . "(json_last_error() was $jsonError)";
          throw new \UnexpectedValueException($msg);
        }

        if ($environment == 'live') {
          $event = \Stripe\Event::retrieve($data['id']);
        } else {
          $event = \Stripe\Event::constructFrom($data, null);
        }
      }
    } catch (\UnexpectedValueException $e) {
      return new Response('Invalid payload', Response::HTTP_BAD_REQUEST);
    } catch (\Stripe\Error\SignatureVerification $e) {
      return new Response('Invalid signature', Response::HTTP_BAD_REQUEST);
    }

    // Dispatch the webhook charge succeeded event.
    $this->event_dispatcher
      ->dispatch(StripeEvents::WEBHOOK_CHARGE_FAILED, new StripeWebhookChargeFailed($event));

    \Drupal::logger('chargeFailed')->notice('Hi chargeFailed' . $event->type);


    return new Response('OK', Response::HTTP_OK);
  }
}
